@@ -37,14 +37,16 @@ import (
 
 	monClient "github.com/coreos/prometheus-operator/pkg/client/versioned/typed/monitoring/v1"
 	"github.com/pkg/errors"
+	admissionclient "k8s.io/client-go/kubernetes/typed/admissionregistration/v1"
 	apiservicesclient "k8s.io/kube-aggregator/pkg/client/clientset_generated/clientset"
 
 	metricsclient "k8s.io/metrics/pkg/client/clientset/versioned"
 )
 
-var (
+const (
 	namespaceName             = "openshift-monitoring"
 	userWorkloadNamespaceName = "openshift-user-workload-monitoring"
+	e2eServiceAccount         = "cluster-monitoring-operator-e2e"
 )
 
 type Framework struct {
@@ -54,6 +56,7 @@ type Framework struct {
 	PrometheusK8sClient *PrometheusClient
 	AlertmanagerClient  *PrometheusClient
 	APIServicesClient   *apiservicesclient.Clientset
+	AdmissionClient     *admissionclient.AdmissionregistrationV1Client
 	MetricsClient       *metricsclient.Clientset
 	kubeConfigPath      string
 
@@ -85,7 +88,7 @@ func New(kubeConfigPath string) (*Framework, cleanUpFunc, error) {
 		return nil, nil, errors.Wrap(err, "creating monitoring client failed")
 	}
 
-	operatorClient, err := client.New(config, "", namespaceName, "")
+	operatorClient, err := client.New(config, "", namespaceName, userWorkloadNamespaceName, "")
 	if err != nil {
 		return nil, nil, errors.Wrap(err, "creating operator client failed")
 	}
@@ -95,6 +98,11 @@ func New(kubeConfigPath string) (*Framework, cleanUpFunc, error) {
 		return nil, nil, errors.Wrap(err, "creating API services client failed")
 	}
 
+	admissionClient, err := admissionclient.NewForConfig(config)
+	if err != nil {
+		return nil, nil, errors.Wrap(err, "creating admission registration client failed")
+	}
+
 	metricsClient, err := metricsclient.NewForConfig(config)
 	if err != nil {
 		return nil, nil, errors.Wrap(err, "creating metrics client failed")
@@ -104,6 +112,7 @@ func New(kubeConfigPath string) (*Framework, cleanUpFunc, error) {
 		OperatorClient:           operatorClient,
 		KubeClient:               kubeClient,
 		APIServicesClient:        apiServicesClient,
+		AdmissionClient:          admissionClient,
 		MetricsClient:            metricsClient,
 		MonitoringClient:         mClient,
 		Ns:                       namespaceName,
@@ -116,7 +125,7 @@ func New(kubeConfigPath string) (*Framework, cleanUpFunc, error) {
 		return nil, nil, errors.Wrap(err, "failed to setup test framework")
 	}
 
-	token, err := f.GetServiceAccountToken("openshift-monitoring", "cluster-monitoring-operator-e2e")
+	token, err := f.GetServiceAccountToken(namespaceName, e2eServiceAccount)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -124,7 +133,7 @@ func New(kubeConfigPath string) (*Framework, cleanUpFunc, error) {
 	// Prometheus client depends on setup above.
 	f.ThanosQuerierClient, err = NewPrometheusClientFromRoute(
 		openshiftRouteClient,
-		"openshift-monitoring", "thanos-querier",
+		namespaceName, "thanos-querier",
 		token,
 	)
 	if err != nil {
@@ -133,7 +142,7 @@ func New(kubeConfigPath string) (*Framework, cleanUpFunc, error) {
 
 	f.PrometheusK8sClient, err = NewPrometheusClientFromRoute(
 		openshiftRouteClient,
-		"openshift-monitoring", "prometheus-k8s",
+		namespaceName, "prometheus-k8s",
 		token,
 	)
 	if err != nil {
@@ -142,7 +151,7 @@ func New(kubeConfigPath string) (*Framework, cleanUpFunc, error) {
 
 	f.AlertmanagerClient, err = NewPrometheusClientFromRoute(
 		openshiftRouteClient,
-		"openshift-monitoring", "alertmanager-main",
+		namespaceName, "alertmanager-main",
 		token,
 	)
 	if err != nil {
@@ -158,14 +167,14 @@ type cleanUpFunc func() error
 func (f *Framework) setup() (cleanUpFunc, error) {
 	cleanUpFuncs := []cleanUpFunc{}
 
-	cf, err := f.CreateServiceAccount(f.Ns, "cluster-monitoring-operator-e2e")
+	cf, err := f.CreateServiceAccount(f.Ns, e2eServiceAccount)
 	if err != nil {
 		return nil, err
 	}
 
 	cleanUpFuncs = append(cleanUpFuncs, cf)
 
-	cf, err = f.CreateClusterRoleBinding(f.Ns, "cluster-monitoring-operator-e2e", "cluster-monitoring-view")
+	cf, err = f.CreateClusterRoleBinding(f.Ns, e2eServiceAccount, "cluster-monitoring-view")
 	if err != nil {
 		return nil, err
 	}
@@ -212,21 +221,26 @@ func (f *Framework) CreateServiceAccount(namespace, serviceAccount string) (clea
 }
 
 func (f *Framework) GetServiceAccountToken(namespace, name string) (string, error) {
-	secrets, err := f.KubeClient.CoreV1().Secrets(namespace).List(context.TODO(), metav1.ListOptions{})
-	if err != nil {
-		return "", err
-	}
-	for _, secret := range secrets.Items {
-		_, dockerToken := secret.Annotations["openshift.io/create-dockercfg-secrets"]
-		token := strings.Contains(secret.Name, fmt.Sprintf("%s-token-", name))
-
-		// we have to skip the token secret that contains the openshift.io/create-dockercfg-secrets annotation
-		// as this is the token to talk to the internal registry.
-		if !dockerToken && token {
-			return string(secret.Data["token"]), nil
+	var token string
+	err := Poll(5*time.Second, time.Minute, func() error {
+		secrets, err := f.KubeClient.CoreV1().Secrets(namespace).List(context.TODO(), metav1.ListOptions{})
+		if err != nil {
+			return err
 		}
-	}
-	return "", errors.Errorf("cannot find token for %s/%s service account", namespace, name)
+		for _, secret := range secrets.Items {
+			// we have to skip the token secret that contains the openshift.io/create-dockercfg-secrets annotation
+			// as this is the token to talk to the internal registry.
+			if _, dockerToken := secret.Annotations["openshift.io/create-dockercfg-secrets"]; dockerToken {
+				continue
+			}
+			if strings.Contains(secret.Name, fmt.Sprintf("%s-token-", name)) {
+				token = string(secret.Data["token"])
+				return nil
+			}
+		}
+		return errors.Errorf("cannot find token for %s/%s service account", namespace, name)
+	})
+	return token, err
 }
 
 func (f *Framework) CreateClusterRoleBinding(namespace, serviceAccount, clusterRole string) (cleanUpFunc, error) {
